@@ -1,7 +1,7 @@
 /******************************************************************************
-	QtAV:  Media play library based on Qt and FFmpeg
+    QtAV:  Multimedia framework based on Qt and FFmpeg
 	solve the version problem and diffirent api in FFmpeg and libav
-    Copyright (C) 2012-2016 Wang Bin <wbsecg1@gmail.com>
+    Copyright (C) 2012-2018 Wang Bin <wbsecg1@gmail.com>
 
 *   This file is part of QtAV
 
@@ -38,7 +38,7 @@
 #define AV_ENSURE(FUNC, ...) AV_RUN_CHECK(FUNC, return, __VA_ARGS__)
 #define AV_WARN(FUNC) AV_RUN_CHECK(FUNC, void)
 
-#include "QtAV_Global.h"
+#include "QtAV/QtAV_Global.h"
 #ifdef __cplusplus
 extern "C"
 {
@@ -59,6 +59,10 @@ extern "C"
 #include <libavutil/parseutils.h>
 #include <libavutil/pixdesc.h>
 #include <libavutil/avstring.h>
+#include <libavfilter/version.h>
+
+#define AVCODEC_STATIC_REGISTER FFMPEG_MODULE_CHECK(LIBAVCODEC, 58, 10, 100)
+#define AVFORMAT_STATIC_REGISTER FFMPEG_MODULE_CHECK(LIBAVFORMAT, 58, 9, 100)
 
 #if !FFMPEG_MODULE_CHECK(LIBAVUTIL, 51, 73, 101)
 #include <libavutil/channel_layout.h>
@@ -79,8 +83,11 @@ extern "C"
 #endif //QTAV_HAVE(AVRESAMPLE)
 
 #if QTAV_HAVE(AVFILTER)
+#if LIBAVFILTER_VERSION_INT < AV_VERSION_INT(3,8,0)
 #include <libavfilter/avfiltergraph.h> /*code is here for old version*/
+#else
 #include <libavfilter/avfilter.h>
+#endif
 #include <libavfilter/buffersink.h>
 #include <libavfilter/buffersrc.h>
 #endif //QTAV_HAVE(AVFILTER)
@@ -132,7 +139,7 @@ int avformat_alloc_output_context2(AVFormatContext **avctx, AVOutputFormat *ofor
 #endif
 //TODO: always inline
 /* --gnu option of the RVCT compiler also defines __GNUC__ */
-#if defined(Q_CC_GNU) && !defined(Q_CC_RVCT)
+#if defined(__GNUC__) && !(defined(__ARMCC__) || defined(__CC_ARM))
 #define GCC_VERSION_AT_LEAST(major, minor, patch) \
     (__GNUC__ > major || (__GNUC__ == major && (__GNUC_MINOR__ > minor \
     || (__GNUC_MINOR__ == minor && __GNUC_PATCHLEVEL__ >= patch))))
@@ -144,10 +151,7 @@ int avformat_alloc_output_context2(AVFormatContext **avctx, AVOutputFormat *ofor
 //FFmpeg2.0, Libav10 2013-03-08 - Reference counted buffers - lavu 52.19.100/52.8.0, lavc 55.0.100 / 55.0.0, lavf 55.0.100 / 55.0.0, lavd 54.4.100 / 54.0.0, lavfi 3.5.0
 #define QTAV_HAVE_AVBUFREF AV_MODULE_CHECK(LIBAVUTIL, 52, 8, 0, 19, 100)
 
-/*TODO: libav
-avutil: error.h
-*/
-#if defined(Q_CC_MSVC) || !defined(av_err2str) || (GCC_VERSION_AT_LEAST(4, 7, 0) && __cplusplus)
+#if defined(_MSC_VER) || !defined(av_err2str) || (GCC_VERSION_AT_LEAST(4, 7, 0) && __cplusplus)
 #ifdef av_err2str
 #undef av_err2str
 /*#define av_make_error_string qtav_make_error_string*/
@@ -170,22 +174,23 @@ static av_always_inline char *av_make_error_string(char *errbuf, size_t errbuf_s
 #endif /*av_err2str*/
 
 #define AV_ERROR_MAX_STRING_SIZE 64
+#ifdef QT_CORE_LIB
+#include <QtCore/QSharedPointer>
+#define av_err2str(e) av_err2str_qsp(e).data()
+av_always_inline QSharedPointer<char> av_err2str_qsp(int errnum)
+{
+    QSharedPointer<char> str((char*)calloc(AV_ERROR_MAX_STRING_SIZE, 1), ::free);
+    av_strerror(errnum, str.data(), AV_ERROR_MAX_STRING_SIZE);
+    return str;
+}
+#else
 av_always_inline char* av_err2str(int errnum)
 {
     static char str[AV_ERROR_MAX_STRING_SIZE];
     memset(str, 0, sizeof(str));
     return av_make_error_string(str, AV_ERROR_MAX_STRING_SIZE, errnum);
 }
-
-/**
- * Convenience macro, the return value should be used only directly in
- * function arguments but never stand-alone.
- */
-/*GCC: taking address of temporary array*/
-/*
-#define av_err2str(errnum) \
-    av_make_error_string((char[AV_ERROR_MAX_STRING_SIZE]){0}, AV_ERROR_MAX_STRING_SIZE, errnum)
-*/
+#endif /* QT_CORE_LIB */
 #endif /*!defined(av_err2str) || GCC_VERSION_AT_LEAST(4, 7, 2)*/
 
 #if (LIBAVCODEC_VERSION_INT <= AV_VERSION_INT(52,23,0))
@@ -390,17 +395,24 @@ int av_packet_copy_props(AVPacket *dst, const AVPacket *src);
 #if !LIBAV_MODULE_CHECK(LIBAVCODEC, 55, 34, 1) && !FFMPEG_MODULE_CHECK(LIBAVCODEC, 55, 39, 100)
 void av_packet_free_side_data(AVPacket *pkt);
 #endif
-
-#if !AV_MODULE_CHECK(LIBAVCODEC, 55, 52, 0, 63, 100)
-void avcodec_free_context(AVCodecContext **avctx);
+//ffmpeg2.1 libav10
+#if !AV_MODULE_CHECK(LIBAVCODEC, 55, 34, 1, 39, 101)
+int av_packet_ref(AVPacket *dst, const AVPacket *src);
+#define av_packet_unref(pkt) av_free_packet(pkt)
 #endif
 
+#if !AV_MODULE_CHECK(LIBAVCODEC, 55, 52, 0, 63, 100)
+void avcodec_free_context(AVCodecContext **pavctx);
+#endif
+
+#if QTAV_HAVE(AVFILTER)
 // ffmpeg2.0 2013-07-03 - 838bd73 - lavfi 3.78.100 - avfilter.h
 #if QTAV_USE_LIBAV(LIBAVFILTER)
 #define avfilter_graph_parse_ptr(pGraph, pFilters, ppInputs, ppOutputs, pLog) avfilter_graph_parse(pGraph, pFilters, *ppInputs, *ppOutputs, pLog)
 #elif !FFMPEG_MODULE_CHECK(LIBAVFILTER, 3, 78, 100)
 #define avfilter_graph_parse_ptr(pGraph, pFilters, ppInputs, ppOutputs, pLog) avfilter_graph_parse(pGraph, pFilters, ppInputs, ppOutputs, pLog)
-#endif
+#endif //QTAV_USE_LIBAV(LIBAVFILTER)
+
 //ffmpeg1.0 2012-06-12 - c7b9eab / 84b9fbe - lavfi 2.79.100 / 2.22.0 - avfilter.h
 #if !AV_MODULE_CHECK(LIBAVFILTER, 2, 22, 0, 79, 100) //FF_API_AVFILTERPAD_PUBLIC
 const char *avfilter_pad_get_name(const AVFilterPad *pads, int pad_idx);
@@ -417,14 +429,49 @@ int avfilter_copy_buf_props(AVFrame *dst, const AVFilterBufferRef *src);
 }
 #endif /* __cplusplus */
 #endif
+#endif //QTAV_HAVE(AVFILTER)
+
 /* helper functions */
 const char *get_codec_long_name(AVCodecID id);
 
+// AV_CODEC_ID_H265 is a macro defined as AV_CODEC_ID_HEVC in ffmpeg but not in libav. so we can use FF_PROFILE_HEVC_MAIN to avoid libavcodec version check. (from ffmpeg 2.1)
+#ifndef FF_PROFILE_HEVC_MAIN //libav does not define it
+#define AV_CODEC_ID_HEVC ((AVCodecID)0) //QTAV_CODEC_ID(NONE)
+#define CODEC_ID_HEVC ((AVCodecID)0) //QTAV_CODEC_ID(NONE)
+#define FF_PROFILE_HEVC_MAIN -1
+#define FF_PROFILE_HEVC_MAIN_10 -1
+#endif
+#if !FFMPEG_MODULE_CHECK(LIBAVCODEC, 54, 92, 100) && !LIBAV_MODULE_CHECK(LIBAVCODEC, 55, 34, 1) //ffmpeg1.2 libav10
+#define AV_CODEC_ID_VP9 ((AVCodecID)0) //QTAV_CODEC_ID(NONE)
+#define CODEC_ID_VP9 ((AVCodecID)0) //QTAV_CODEC_ID(NONE)
+#endif
+#ifndef FF_PROFILE_VP9_0
+#define FF_PROFILE_VP9_0 0
+#define FF_PROFILE_VP9_1 1
+#define FF_PROFILE_VP9_2 2
+#define FF_PROFILE_VP9_3 3
+#endif
+
 #define AV_RUN_CHECK(FUNC, RETURN, ...) do { \
     int ret = FUNC; \
     if (ret < 0) { \
-        av_log(NULL, AV_LOG_WARNING, "Error " #FUNC " @%d " __FILE__ ": (%#x) %s\n", __LINE__, ret, av_err2str(ret)); \
+        char str[AV_ERROR_MAX_STRING_SIZE]; \
+        memset(str, 0, sizeof(str)); \
+        av_strerror(ret, str, sizeof(str)); \
+        av_log(NULL, AV_LOG_WARNING, "Error " #FUNC " @%d " __FILE__ ": (%#x) %s\n", __LINE__, ret, str); \
         RETURN __VA_ARGS__; \
      } } while(0)
 
 #endif //QTAV_COMPAT_H
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(56,33,0)
+#define AV_CODEC_FLAG_GLOBAL_HEADER CODEC_FLAG_GLOBAL_HEADER
+#endif
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(56,56,100)
+#define AV_INPUT_BUFFER_MIN_SIZE FF_MIN_BUFFER_SIZE
+#endif
+
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(56,56,100)
+#define AV_INPUT_BUFFER_PADDING_SIZE FF_INPUT_BUFFER_PADDING_SIZE
+#endif
